@@ -60,7 +60,7 @@ soup_xmlrpc_response_init (SoupXmlrpcResponse *response)
 {
 	SoupXmlrpcResponsePrivate *priv = SOUP_XMLRPC_RESPONSE_GET_PRIVATE (response);
 
-	priv->doc = xmlNewDoc ((xmlChar *)"1.0");
+	priv->doc = xmlNewDoc ((const xmlChar *)"1.0");
 	priv->fault = FALSE;
 }
 
@@ -94,20 +94,11 @@ soup_xmlrpc_response_new_from_string (const char *xmlstr)
 static xmlNode *
 exactly_one_child (xmlNode *node)
 {
-	xmlNode *child, *tmp;
+	xmlNode *child;
 
-	tmp = node->children;
-	while (tmp && xmlIsBlankNode (tmp))
-		tmp = tmp->next;
-
-	child = tmp;
-	if (tmp && tmp->next) {
-		tmp = tmp->next;
-		while (tmp && xmlIsBlankNode (tmp))
-			tmp = tmp->next;
-		if (tmp)
-			return NULL;
-	}
+	child = soup_xml_real_node (node->children);
+	if (!child || soup_xml_real_node (child->next))
+		return NULL;
 
 	return child;
 }
@@ -124,29 +115,28 @@ soup_xmlrpc_response_from_string (SoupXmlrpcResponse *response, const char *xmls
 	priv = SOUP_XMLRPC_RESPONSE_GET_PRIVATE (response);
 	g_return_val_if_fail (xmlstr != NULL, FALSE);
 
-	xmlKeepBlanksDefault (0);
 	newdoc = xmlParseMemory (xmlstr, strlen (xmlstr));
 	if (!newdoc)
 		goto very_bad;
 
 	body = xmlDocGetRootElement (newdoc);
-	if (!body || strcmp ((char *)body->name, "methodResponse"))
+	if (!body || strcmp ((const char *)body->name, "methodResponse"))
 		goto bad;
 
 	body = exactly_one_child (body);
 	if (!body)
 		goto bad;
 
-	if (strcmp ((char *)body->name, "params") == 0) {
+	if (strcmp ((const char *)body->name, "params") == 0) {
 		fault = FALSE;
 		body = exactly_one_child (body);
-		if (!body || strcmp ((char *)body->name, "param"))
+		if (!body || strcmp ((const char *)body->name, "param"))
 			goto bad;
-	} else if (strcmp ((char *)body->name, "fault") != 0)
+	} else if (strcmp ((const char *)body->name, "fault") != 0)
 		goto bad;
 
 	body = exactly_one_child (body);
-	if (!body || strcmp ((char *)body->name, "value"))
+	if (!body || strcmp ((const char *)body->name, "value"))
 		goto bad;
 
 	/* body should be pointing by now to the struct of a fault, or the value of a
@@ -206,28 +196,28 @@ soup_xmlrpc_value_get_type (SoupXmlrpcValue *value)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return SOUP_XMLRPC_VALUE_TYPE_BAD;
 
 	xml = exactly_one_child (xml);
 	if (!xml)
 		return SOUP_XMLRPC_VALUE_TYPE_BAD;
 
-	if (strcmp ((char *)xml->name, "i4") == 0 || strcmp ((char *)xml->name, "int") == 0)
+	if (strcmp ((const char *)xml->name, "i4") == 0 || strcmp ((const char *)xml->name, "int") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_INT;
-	else if (strcmp ((char *)xml->name, "boolean") == 0)
+	else if (strcmp ((const char *)xml->name, "boolean") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_BOOLEAN;
-	else if (strcmp ((char *)xml->name, "string") == 0)
+	else if (strcmp ((const char *)xml->name, "string") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_STRING;
-	else if (strcmp ((char *)xml->name, "double") == 0)
+	else if (strcmp ((const char *)xml->name, "double") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_DOUBLE;
-	else if (strcmp ((char *)xml->name, "dateTime.iso8601") == 0)
+	else if (strcmp ((const char *)xml->name, "dateTime.iso8601") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_DATETIME;
-	else if (strcmp ((char *)xml->name, "base64") == 0)
+	else if (strcmp ((const char *)xml->name, "base64") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_BASE64;
-	else if (strcmp ((char *)xml->name, "struct") == 0)
+	else if (strcmp ((const char *)xml->name, "struct") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_STRUCT;
-	else if (strcmp ((char *)xml->name, "array") == 0)
+	else if (strcmp ((const char *)xml->name, "array") == 0)
 		return SOUP_XMLRPC_VALUE_TYPE_ARRAY;
 	else
 		return SOUP_XMLRPC_VALUE_TYPE_BAD;
@@ -243,10 +233,10 @@ soup_xmlrpc_value_get_int (SoupXmlrpcValue *value, long *i)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || (strcmp ((char *)xml->name, "int") && strcmp ((char *)xml->name, "i4")))
+	if (!xml || (strcmp ((const char *)xml->name, "int") && strcmp ((const char *)xml->name, "i4")))
 		return FALSE;
 
 	/* FIXME this should be exactly one text node */
@@ -268,10 +258,10 @@ soup_xmlrpc_value_get_double (SoupXmlrpcValue *value, double *d)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || (strcmp ((char *)xml->name, "double")))
+	if (!xml || (strcmp ((const char *)xml->name, "double")))
 		return FALSE;
 
 	/* FIXME this should be exactly one text node */
@@ -294,10 +284,10 @@ soup_xmlrpc_value_get_boolean (SoupXmlrpcValue *value, gboolean *b)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || strcmp ((char *)xml->name, "boolean"))
+	if (!xml || strcmp ((const char *)xml->name, "boolean"))
 		return FALSE;
 
 	content = xmlNodeGetContent (xml);
@@ -317,10 +307,10 @@ soup_xmlrpc_value_get_string (SoupXmlrpcValue *value, char **str)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || strcmp ((char *)xml->name, "string"))
+	if (!xml || strcmp ((const char *)xml->name, "string"))
 		return FALSE;
 
 	content = xmlNodeGetContent (xml);
@@ -338,10 +328,10 @@ soup_xmlrpc_value_get_datetime (SoupXmlrpcValue *value, time_t *timeval)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || (strcmp ((char *)xml->name, "dateTime.iso8601")))
+	if (!xml || (strcmp ((const char *)xml->name, "dateTime.iso8601")))
 		return FALSE;
 
 	/* FIXME this should be exactly one text node */
@@ -356,24 +346,29 @@ soup_xmlrpc_value_get_datetime (SoupXmlrpcValue *value, time_t *timeval)
 	return TRUE;
 }
 
-/* FIXME: this is broken; it returns the encoded value, not decoded */
 gboolean
-soup_xmlrpc_value_get_base64 (SoupXmlrpcValue *value, char **buf)
+soup_xmlrpc_value_get_base64 (SoupXmlrpcValue *value, GByteArray **data)
 {
 	xmlNode *xml;
 	xmlChar *content;
+	char *decoded;
+	int len;
 
 	xml = (xmlNode *) value;
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
-	if (!xml || strcmp ((char *)xml->name, "base64"))
+	if (!xml || strcmp ((const char *)xml->name, "base64"))
 		return FALSE;
 
 	content = xmlNodeGetContent (xml);
-	*buf = content ? g_strdup ((char *)content) : g_strdup ("");
+	decoded = soup_base64_decode ((const char *)content, &len);
 	xmlFree (content);
 
+	*data = g_byte_array_new ();
+	g_byte_array_append (*data, (guchar *)decoded, len);
+	g_free (decoded);
+
 	return TRUE;
 }
 
@@ -386,43 +381,50 @@ soup_xmlrpc_value_get_struct (SoupXmlrpcValue *value, GHashTable **table)
 
 	xml = (xmlNode *) value;
 
-	if (strcmp ((char *)xml->name, "value"))
+	if (strcmp ((const char *)xml->name, "value"))
 		return FALSE;
 	xml = exactly_one_child (xml);
 
-	if (!xml || strcmp ((char *)xml->name, "struct"))
+	if (!xml || strcmp ((const char *)xml->name, "struct"))
 		return FALSE;
 
 	t = g_hash_table_new_full (g_str_hash, g_str_equal, xmlFree, NULL);
 
-	for (xml = xml->children; xml; xml = xml->next) {
+	for (xml = soup_xml_real_node (xml->children);
+	     xml;
+	     xml = soup_xml_real_node (xml->next)) {
 		xmlChar *name;
 		xmlNode *val, *cur;
 
-		if (strcmp ((char *)xml->name, "member") || !xml->children)
+		if (strcmp ((const char *)xml->name, "member") ||
+		    !xml->children)
 			goto bad;
 
 		name = NULL;
 		val = NULL;
 
-		for (cur = xml->children; cur; cur = cur->next) {
-			if (strcmp((char *)cur->name, "name") == 0) {
+		for (cur = soup_xml_real_node (xml->children);
+		     cur;
+		     cur = soup_xml_real_node (cur->next)) {
+			if (strcmp((const char *)cur->name, "name") == 0) {
 				if (name)
 					goto local_bad;
 				name = xmlNodeGetContent (cur);
-			}
-			else if (strcmp ((char *)cur->name, "value") == 0)
+			} else if (strcmp ((const char *)cur->name, "value") == 0)
 				val = cur;
-			else goto local_bad;
+			else
+				goto local_bad;
 
 			continue;
 local_bad:
-			if (name) xmlFree (name);
+			if (name)
+				xmlFree (name);
 			goto bad;
 		}
 
 		if (!name || !val) {
-			if (name) xmlFree (name);
+			if (name)
+				xmlFree (name);
 			goto bad;
 		}
 		g_hash_table_insert (t, name, val);
@@ -439,15 +441,21 @@ bad:
 gboolean
 soup_xmlrpc_value_array_get_iterator (SoupXmlrpcValue *value, SoupXmlrpcValueArrayIterator **iter)
 {
-	xmlNode *xml;
+	xmlNode *xml, *array, *data;
 
 	xml = (xmlNode *) value;
 
-	if (!xml->children || strcmp((char *)xml->children->name, "data") == 0 || xml->children->next)
+	array = soup_xml_real_node (xml->children);
+	if (!array || strcmp((const char *)array->name, "array") != 0 ||
+	    soup_xml_real_node (array->next))
 		return FALSE;
 
-	*iter = (SoupXmlrpcValueArrayIterator *) xml->children;
+	data = soup_xml_real_node (array->children);
+	if (!data || strcmp((const char *)data->name, "data") != 0 ||
+	    soup_xml_real_node (data->next))
+		return FALSE;
 
+	*iter = (SoupXmlrpcValueArrayIterator *) soup_xml_real_node (data->children);
 	return TRUE;
 }
 
@@ -455,11 +463,15 @@ soup_xmlrpc_value_array_get_iterator (SoupXmlrpcValue *value, SoupXmlrpcValueArr
 SoupXmlrpcValueArrayIterator *
 soup_xmlrpc_value_array_iterator_prev (SoupXmlrpcValueArrayIterator *iter)
 {
-	xmlNode *xml;
+	xmlNode *xml, *prev;
 
 	xml = (xmlNode *) iter;
 
-	return (SoupXmlrpcValueArrayIterator *) xml->prev;
+	prev = xml->prev;
+	while (prev != soup_xml_real_node (prev))
+		prev = prev->prev;
+
+	return (SoupXmlrpcValueArrayIterator *) prev;
 }
 
 SoupXmlrpcValueArrayIterator *
@@ -469,24 +481,14 @@ soup_xmlrpc_value_array_iterator_next (SoupXmlrpcValueArrayIterator *iter)
 
 	xml = (xmlNode *) iter;
 
-	return (SoupXmlrpcValueArrayIterator *) xml->next;
+	return (SoupXmlrpcValueArrayIterator *) soup_xml_real_node (xml->next);
 }
 
 gboolean
 soup_xmlrpc_value_array_iterator_get_value (SoupXmlrpcValueArrayIterator *iter,
 					    SoupXmlrpcValue **value)
 {
-	xmlNode *xml;
-
-	xml = (xmlNode *) iter;
-
-	if (!xml || strcmp((char *)xml->name, "data"))
-		return FALSE;
-	xml = exactly_one_child (xml);
-	if (!xml)
-		return FALSE;
-
-	*value = (SoupXmlrpcValue *) xml;
+	*value = (SoupXmlrpcValue *) iter;
 
 	return TRUE;
 }
@@ -525,6 +527,7 @@ soup_xmlrpc_value_dump_internal (SoupXmlrpcValue *value, int d)
 	char *str;
 	double f;
 	time_t timeval;
+	GByteArray *base64;
 	GHashTable *hash;
 	SoupXmlrpcValueArrayIterator *iter;
 
@@ -580,10 +583,19 @@ soup_xmlrpc_value_dump_internal (SoupXmlrpcValue *value, int d)
 
 		case SOUP_XMLRPC_VALUE_TYPE_BASE64:
 			indent (d);
-			if (!soup_xmlrpc_value_get_base64 (value, &str))
+			if (!soup_xmlrpc_value_get_base64 (value, &base64))
 				g_printerr ("BAD BASE64\n");
-			else
-				g_printerr ("BASE64: %s\n", str);
+			else {
+				GString *hex = g_string_new (NULL);
+				int i;
+
+				for (i = 0; i < base64->len; i++)
+					g_string_append_printf (hex, "%02x", base64->data[i]);
+
+				g_printerr ("BASE64: %s\n", hex->str);
+				g_string_free (hex, TRUE);
+				g_byte_array_free (base64, TRUE);
+			}
 
 			break;
 
